import os
from random import randrange
import pygame
from pygame.locals import *

from pgu import engine, tilevid, ani

W,H = 20,15
TW,TH = 32,32
SW,SH = 640,480
FPS = 40

HL = 10 #max hair length
GF = FPS*20 #how many frames takes for hair to grown 1 pixel
CL = 0 #length of cut hair

class Main(engine.State):
    def __init__(self,game):
        self.game = game
        
    def init(self):
        tv = self.tv = tilevid.Tilevid()
        
        tv.tga_load_tiles(os.path.join("data","levels","tiles.tga"),(TW,TH),TDATA)
        img = pygame.Surface((TW,TH),32).convert_alpha()
        img.fill((0,0,0,0))
        tv.tiles[0].image = img
        tv.tiles[1].image = img
        tv.tiles[3].image = img
        tv.tga_load_level(os.path.join("data","levels","%d.tga"%((self.game.round%5)+1)))
        tv.view = pygame.Rect(0,0,SW,SH)
        
        img = pygame.image.load(os.path.join("data","gfx","mower.png"))
        ani.ani_load(tv,"player",img,(48,96),(8,8,32,32),[4])
        
        for n in xrange(0,4):
            ani.image_rotate(tv,"player.%d"%n,tv.images["player.%d"%n][0],tv.images["player.%d"%n][1],[0,90,180,270],-90)
        
        tv.run_codes(CDATA,(0,0,tv.size[0],tv.size[1]))
        
        self.bkgr = pygame.image.load(os.path.join("data","gfx","philhassey.jpg")).convert()
        
        self.hairs = Hairs(tv)
        tv.hairs = self.hairs
        
        tv.frame = 0
        
        self.timer = FPS*30
        self.percent = 0
        
        self.game.sfx['shavingtime'].play()
        
        self.noise = 0
        tv.game = self.game
        return Grow(self.game,self)        
        
    def paint(self,screen):
        tv = self.tv
        screen.blit(self.bkgr,(0,0))
        screen.blit(self.hairs.image,(0,0))
        
        #tv.paint(screen)
        self.mypaint(screen)
        
        fnt = self.game.font_score
        
        img = fnt.render("%05d"%self.game.score,1,(255,255,255))
        screen.blit(img,(SW-(8+img.get_width()),(SH-(img.get_height()))))
        
        """
        p = tv.player
        
        if p.gas > 0:
            pad = 8
            size = 16
            c = (0,255,0)
            if p.gas < FPS*3:
                if (tv.frame/2)%2 == 0: c = (255,0,255)
            screen.fill(c,(pad,SH-(size+pad),p.gas,size))
        """
        
        g = self.game
        c = (255,255,255)
        pad = 8
        
        #img = fnt.render("Credits: %02d"%23,1,c)
        #screen.blit(img,(SW-(pad+img.get_width()),pad))
        
        
        self.status(screen,"Time",max(0,(self.timer/FPS)),30,(255,255,255),(pad,pad))
        self.status(screen,"Complete",self.complete,100,(0,255,0),(pad,SH-(pad+60)))
        self.status(screen,"Botched",self.botched,100,(255,0,0),(pad,SH-(pad+30)))
        pygame.display.flip()
        
    def status(self,screen,text,value,top,color,pos):
        fnt = self.game.font
        text = "%s: %02d"%(text,value)
        c = (255,255,255)
        img = fnt.render(text,1,c)
        
        r = pygame.Rect(pos[0]+160,pos[1]+12,(SW-(8+pos[0]+160))*value/top,15)
        
        screen.blit(img,pos)
        screen.fill(color,r)
        pygame.draw.rect(screen,(0,0,0),r,1)
        
        
        
    def mypaint(self,screen):    
        tv = self.tv
        tlayer = tv.tlayer
        tiles = tv.tiles
        blit = screen.blit
        for ty in xrange(0,H):
            tr = tlayer[ty]
            y = ty*TH
            for tx in xrange(0,W):
                n = tr[tx]
                if n >=0: blit(tiles[n].image,(tx*TW,y))
                
        for s in tv.sprites:
            s.irect.x = s.rect.x-s.shape.x
            s.irect.y = s.rect.y-s.shape.y
            blit(s.image,(s.irect.x,s.irect.y))
                


    def loop(self):
        if self.timer <= 0:
            return Score(self.game,self)
            
        if not self.noise:
            self.noise = 1
            #self.game.sfx['shaving'].play(-1)
            if self.game.audio:
                pygame.mixer.music.load(os.path.join("data","music","level.ogg"))
                pygame.mixer.music.play(-1)
                pygame.mixer.music.set_volume(0.5)

        tv = self.tv
        tv.loop()
        tv.frame += 1
        
        self.hairs.grow(tv.frame)
        
        #print self.hairs.total,self.hairs.diffs
        
        self.timer -= 1
        
        c = self.hairs.diffs[0]*100/self.hairs.total[0]
        
        self.complete = min(100,c*100/100) #let 'em think they are done...
        self.botched = self.hairs.diffs[1]*100/self.hairs.total[1]
        self.percent = self.complete-self.botched
            
    def update(self,screen):
        return self.paint(screen)
        
    def event(self,e):
        tv = self.tv
        p = tv.player
        kmap = {
            K_UP:(90,(0,-1)),
            K_DOWN:(270,(0,1)),
            K_LEFT:(180,(-1,0)),
            K_RIGHT:(0,(1,0)),
            }
        if e.type is KEYDOWN:
            if e.key == K_ESCAPE:
                import menu
                return menu.Main(self.game)
            if e.key in kmap:
                facing,dirs = kmap[e.key]
                p.facing,p.dirs = facing,dirs
        if e.type is KEYUP:
            if e.key in kmap:
                facing,dirs = kmap[e.key]
                if p.dirs == dirs: p.dirs = (0,0)
        
def player_new(tv,t,v):
    s = tilevid.Sprite(tv.images['player.0'],t.rect)


    s.groups = tv.string2groups("player")
    s.frame = 0 
    s.facing = 90
    s.dirs = (0,0)
    s.loop = player_loop
    s.v = 4
    s.gas = FPS*15
            
    tv.player = s
    tv.sprites.append(s)

def player_loop(tv,s):
    dirs = s.dirs
    v = s.v
    s.rect.x += dirs[0]*v
    s.rect.y += dirs[1]*v
    
    s.rect.clamp_ip(tv.view)
    
    s.frame += 1
    if (s.frame%2)==0: s.gas -= 1
    
    #tv.hairs.image.fill((0,0,0,0),s.rect)
    tv.hairs.trim(s.rect,tv.frame)
    
    s.setimage(tv.images['player.%d.%d'%((s.frame/3)%4,s.facing)])
    
    tx,ty = s.rect.centerx/TW,s.rect.centery/TH
    if tv.clayer[ty][tx] == 0x02:
        if tv.game.audio:
            c = pygame.mixer.Channel(0)
            if not c.get_busy():
                c.play(tv.game.sfx['nipple'])

class Grow(engine.State):
    def __init__(self,game,next):
        self.game = game
        self.next = next
        
    def init(self):
        self.frame = 0
        self.hairs = self.next.hairs
        self.bkgr = self.next.bkgr
    
    def paint(self,screen):
        screen.blit(self.bkgr,(0,0))
        pygame.display.flip()
        
    def update(self,screen):
        hairs = self.hairs
        f = self.frame
        if (f%4)==0:
            for ty in xrange(0,H):
                for tx in xrange(0,W):
                    hairs.alayer[ty][tx] = 1
                    for h in hairs.layer[ty][tx]:
                        h.l += 1
            hairs.update()
            screen.blit(hairs.image,(0,0))
            pygame.display.flip()
    
        self.frame += 1
        if self.frame == HL*4: 
            self.game.sfx['revup'].play()
            return self.next
    
        
class Hair:
    pass
        
class Hairs:
    def __init__(self,tv):
        self.tv = tv
        self.layer = [[[] for tx in xrange(0,W)] for ty in xrange(0,H)]
        self.alayer = [[0 for tx in xrange(0,W)] for ty in xrange(0,H)]
        self.image = pygame.Surface((SW,SH),32).convert_alpha()
        self.image.fill((0,0,0,0))
        self.hairs = []
        self.total = [0,0,0]
        self.diffs = [0,0,0]
        
        self.player = tv.player

        for ty in xrange(0,H):
            for tx in xrange(0,W):
                if tv.tlayer[ty][tx] == 0: continue
                for i in xrange(0,24):
                    xx,yy=randrange(0,TW),randrange(0,TH)
                    x,y,p = tx*TW+xx,ty*TH+yy,randrange(0,8)
                    h = Hair()
                    h.x,h.y,h.p,h.l = x,y,p,0
                    h.tx,h.ty = tx,ty
                    h.frame = None
                    h.n = int(tv.tlayer[ty][tx]>1)
                    pad = 6
                    if xx < pad or xx >= (TW-pad) or yy < pad or yy >= (TH-pad):
                        h.n = 2 #don't count the borders, so people can cut sort of close
                    
                    self.total[h.n] += HL
                        
                    self.layer[ty][tx].append(h)
                    self.hairs.append(h)
        #print 'hairs',len(self.hairs)
        self.togrow = {}
        
    def trim(self,rect,frame):
        layer,alayer,togrow = self.layer,self.alayer,self.togrow
        
        nframe = frame + GF
        if nframe not in togrow: togrow[nframe] = []
        
        for y in xrange((rect.top/TH)*TH,rect.bottom,TH):
            for x in xrange((rect.left/TW)*TW,rect.right,TW):
                tx,ty = x/TW,y/TH
                a = 0
                for h in layer[ty][tx]:
                    if rect.collidepoint((h.x,h.y)):
                        old_l = h.l
                        
                        if h.l > 0: a = 1
                        
                        h.l = CL
                        if h.frame and h.frame in togrow and h in togrow[h.frame]:
                            togrow[h.frame].remove(h)
                        h.frame = nframe
                        togrow[h.frame].append(h)
                        
                        new_l = h.l
                        
                        if h.n == 1:
                            self.tv.game.sfx['shaving'].play()
                            tx,ty = h.x/TW,h.y/TH
                            self.tv.tlayer[ty][tx] = randrange(6,10)
                        
                        self.diffs[h.n] += old_l-new_l
                
                if a:
                    alayer[ty][tx] = 1
                    self.image.fill((0,0,0,0),(tx*TW-3,ty*TH-HL/2,TW+6,TH+HL))
                    for dx,dy in [(-1,-1),(0,-1),(1,-1),(-1,0),(1,0),(-1,1),(0,1),(1,1)]:
                        alayer[ty+dy][tx+dx] = 1
        self.paint()
                
    
    def grow(self,frame):
        alayer,togrow = self.alayer,self.togrow
        
        nframe = frame + GF
        if nframe not in togrow: togrow[nframe] = []
        
        if frame in togrow:
            for h in togrow[frame]:
                if h.l < HL:
                    old_l = h.l
                    h.l += 1
                    alayer[h.ty][h.tx] = 1
                    if h.l < HL:
                        h.frame = nframe
                        togrow[h.frame].append(h)
                    else:
                        h.frame = None
                    new_l = h.l
                    self.diffs[h.n] += old_l - new_l

            del togrow[frame]
        self.update()
                            
        
    def paint(self):
        img = self.image
        for ty in xrange(0,H):
            for tx in xrange(0,W):
                if self.alayer[ty][tx]:
                    for h in self.layer[ty][tx]:
                        x,y,p = h.x,h.y,h.p
                        for l in xrange(1,h.l+1):
                            img.set_at((x+Hairs.paths[p][(l/2)],y+l-HL/2),(0,0,0))
                    self.alayer[ty][tx] = 0
    
    def update(self):
        img = self.image
        for ty in xrange(0,H):
            for tx in xrange(0,W):
                if self.alayer[ty][tx]:
                    for h in self.layer[ty][tx]:
                        x,y,p,l = h.x,h.y,h.p,h.l
                        if l > 0:
                            img.set_at((x+Hairs.paths[p][(l/2)],y+l-HL/2),(0,0,0))
                    self.alayer[ty][tx] = 0

    paths = [
        [ 0,-1,-1,-2,-2,-1, 0, 1],
        [ 0, 1, 0, 1, 2, 1, 1, 2],
        [ 0, 0, 0, 1, 2, 3, 2, 3],
        [ 0,-1,-2,-1,-2,-2,-1, 0],
        [ 0, 1, 1, 2, 2, 1, 0,-1],
        [ 0,-1, 0,-1,-2,-1,-1,-2],
        [ 0, 0, 0,-1,-2,-3,-2,-3],
        [ 0, 1, 2, 1, 2, 2, 1, 0],
        ]
        
    

CDATA = {
    0x01:(player_new,None),
    }

TDATA = {}

class Score(engine.State):
    def __init__(self,game,level):
        self.game = game
        self.level = level
        self.score = self.level.complete * 100 - self.level.botched * 100
        self.game.score += self.score
        self.game.high = max(self.game.high,self.game.score)
        
    def paint(self,screen):
    
        self.game.sfx['yeah'].play()

        fnt = self.game.font_score
        
        img = fnt.render("Score for Level %d: %05d"%(self.game.round+1,self.score))
        
        y = 100
        screen.blit(img,(320-img.get_width()/2,y))
        y+= 60
        
        img = fnt.render("Total Score: %05d"%(self.game.score))
        screen.blit(img,(320-img.get_width()/2,y))
        y += 60
        
        if self.game.round == 4:
            y += 30
            img = fnt.render("You Win!")
            screen.blit(img,(320-img.get_width()/2,y))
            y += 60
            
        y += 30
        img = fnt.render("Press enter to continue")
        screen.blit(img,(320-img.get_width()/2,y))
        y += 60
           
        pygame.display.flip()
        
    def event(self,e):
        if e.type is KEYDOWN and e.key == K_RETURN:
            self.game.round += 1
            if self.game.round < 5:
                return Main(self.game)
            else:
                import menu
                return menu.Main(self.game)