import os
import container, group, input, container, button
import label as labelz
import image as imagez
import table, dialog
from slider import *
#from __init__ import theme
#import app

#===============================================================================
# Added by Gal Koren
#===============================================================================

#class HScroller(table.Table):
#    def __init__(self, slider):
#        table.Table.__init__(self)
#        self.slider = slider
#        img_left = pygame.image.load(os.path.join('data', 'themes', 'default', 'hscrollbar.left.png'))
#        img_right = pygame.image.load(os.path.join('data', 'themes', 'default', 'hscrollbar.right.tga'))
#        self.tr()
#        self.td(imagez.Image(img_left))
#        self.td(self.slider)
#        self.td(imagez.Image(img_right))

class ProxySurface:
    """
    A surface-like object which smartly handle out-of-area blitting.
    
    @ivar mysubsurface: a real and valid pygame.Surface object to be used
                       for blitting.
    @ivar x, y: if the proxy surface is lefter or higher than the parent,
                x, y hold the diffs.
    @ivar offset: an optional feature which let you scroll the whole blitted
                  content.
    """
    def __init__(self, parent, rect, real_surface=None, offset=(0, 0)):
        """
        only one of parent and real_surface should be supplied (non None)
        @param parent: a ProxySurface object
        @param real_surface: a pygame Surface object
        """
        self.offset = offset
        self.x = self.y = 0
        if rect[0] < 0: self.x = rect[0]
        if rect[1] < 0: self.y = rect[1]
        if real_surface == None:
            self.mysubsurface = parent.mysubsurface.subsurface(parent.mysubsurface.get_rect().clip(rect))
        else:
            self.mysubsurface = real_surface.subsurface(real_surface.get_rect().clip(rect))
        rect[0], rect[1] = 0, 0
        self.rect = rect
        
    def blit(self, s, pos, rect=None):
        if rect == None: rect = s.get_rect()
        pos = (pos[0] + self.offset[0] + self.x, pos[1] + self.offset[1] + self.y)
        self.mysubsurface.blit(s, pos, rect)
        
    def subsurface(self, rect): return ProxySurface(self, pygame.Rect(rect).move(self.offset[0] + self.x, self.offset[1] + self.y))
    def fill(self, color, rect=None): 
        if rect != None: self.mysubsurface.fill(color, rect)
        else: self.mysubsurface.fill(color)
    def get_rect(self): return self.rect
    def get_width(self): return self.rect[2]
    def get_height(self): return self.rect[3]
    def get_abs_offset(): return self.rect[:2]
    def get_abs_parent(): return self.mysubsurface.get_abs_parent()
    def set_clip(self, rect=None): 
        if rect == None: self.mysubsurface.set_clip()
        else: 
            rect = [rect[0] + self.offset[0] + self.x, rect[1] + self.offset[0] + self.y, rect[2], rect[3]]
            self.mysubsurface.set_clip(rect)
        


class SlideBox(container.Container):
    """
    a scrollable box with no scrollbars
    """
    
    def __init__(self, w, width, height, **params):
        params.setdefault('width', width)
        params.setdefault('height', height)
        container.Container.__init__(self, **params)
        self.offset = [0, 0]
        self.widget = w
        self.add(w, 0, 0)
    
    def paint(self, s):
        container.Container.paint(self, ProxySurface(parent=None, 
                                           rect=self.max_rect,
                                           real_surface=s,
                                           offset=self.offset))
    def update(self, s):
        rects = container.Container.update(self, ProxySurface(parent=None, 
                                                    rect=self.max_rect,
                                                    real_surface=s,
                                                    offset=self.offset))
        result = []
        for r in rects: result.append(pygame.Rect(r).move(self.offset))
        return result
        
    def resize(self, width=None, height=None):
        container.Container.resize(self)
        self.max_rect = self.widget.rect
        return self.style.width,self.style.height
        #self.rect = pygame.Rect(self.rect[0], self.rect[1], self.style.width, self.style.height)
    
    def event(self, e):
        if e.type in [MOUSEBUTTONDOWN, MOUSEBUTTONUP, MOUSEMOTION]:
            pos = (e.pos[0] - self.offset[0], e.pos[1] - self.offset[1])
            if self.max_rect.collidepoint(pos):
                e_params = {'pos': pos }
                if e.type == MOUSEMOTION: 
                    e_params['buttons'] = e.buttons
                    e_params['rel'] = e.rel
                else:
                    e_params['button'] = e.button
                e = pygame.event.Event(e.type, e_params)
        container.Container.event(self, e)


    
class ScrollArea(table.Table):
    
    def __init__(self, w, width, height, hscrollbar=True, **params):
        params.setdefault('cls', 'scrollarea')
        table.Table.__init__(self, **params)

        #
        # adding the slide box
        #
        self.tr()
        self.widget = w
        self.sbox = SlideBox(w, width=width, height=height, cls=self.cls+".content")
        self.td(self.sbox)
        #
        # adding the scrollbars
        #
        #st, sr, sb, sl = self.sbox.getspacing()
        self.vscrollbar = VScrollBar(0, 0, 1000, size=20, height=self.sbox.rect.h)
        self.td(self.vscrollbar)
        self.vscrollbar.connect(CHANGE, self._vscrollbar_changed, None)
        self.hscrollbar = None
        if not hscrollbar:
            self.hscrollbar = HScrollBar(0, 0, 1000, size=20, width=self.sbox.rect.w)
            self.hscrollbar.connect(CHANGE, self._hscrollbar_changed, None)
            self.tr()
            self.td(self.hscrollbar, colspan=2, align=-1)
    
    def resize(self, width=None, height=None):
        w,h = table.Table.resize(self, width, height)
        if self.hscrollbar:
            if self.widget.rect.w <= self.sbox.rect.w:
                self.hscrollbar.size = self.hscrollbar.style.width
            else:
                self.hscrollbar.size = max(20,self.hscrollbar.style.width * self.sbox.rect.w / self.widget.rect.w)
            self._hscrollbar_changed(None)
        if self.widget.rect.h <= self.sbox.rect.h: 
            self.vscrollbar.size = self.vscrollbar.style.height
        else:
            self.vscrollbar.size = max(20,self.vscrollbar.style.height * self.sbox.rect.h / self.widget.rect.h)
        self._vscrollbar_changed(None)
        return w,h

    def _vscrollbar_changed(self, xxx):
        y = (self.widget.rect.h - self.sbox.rect.h) * self.vscrollbar.value / 1000
        if y >= 0: self.sbox.offset[1] = -y
        self.sbox.repaint()

    def _hscrollbar_changed(self, xxx):
        x = (self.widget.rect.w - self.sbox.rect.w) * self.hscrollbar.value / 1000
        if x >= 0: self.sbox.offset[0] = -x
        self.sbox.repaint()

    def set_vertical_scroll(self, percents): 
        self.vscrollbar.value = min(max(percents*10, 0), 1000)
        self._vscrollbar_changed(None)

    def set_horizontal_scroll(self, percents): 
        self.hscrollbar.value = min(max(percents*10, 0), 1000)
        self._hscrollbar_changed(None)
        
        



class _List_Item(button._button):
    def __init__(self,label=None,image=None,value=None,**params): #TODO label= could conflict with the module label
        """
        @param image: an imagez.Image object (optional)
        @param text: a string object
        """
        params.setdefault('cls','list.item')
        button._button.__init__(self,**params)
        self.group = None
        self.value = value #(self, value)
        self.widget = None
        
        if type(label) == str: 
            label = labelz.Label(label, cls=self.cls+".label")

        if image and label:
            self.widget = container.Container()
            self.widget.add(image, 0, 0)
            image.rect.w,image.rect.h = image.resize()
            self.widget.add(label, image.rect.w, 0)
        elif image: self.widget = image
        elif label: self.widget = label
        
        self.pcls = ""
    
    def resize(self,width=None,height=None):
         self.widget.rect.w,self.widget.rect.h = self.widget.resize()
         return self.widget.rect.w,self.widget.rect.h
#         self.widget._resize()
#         self.rect.w,self.rect.h = self.widget.rect_margin.w,self.widget.rect_margin.h
    
    def event(self,e):
        button._button.event(self,e)
        if self.group.value == self.value: self.pcls = "down"
    
    def paint(self,s):
        if self.group.value == self.value: self.pcls = "down"
        self.widget.paint(mysubsurface(s,self.widget.rect))
    
    def click(self):
        self.group.value = self.value
        for w in self.group.widgets:
            if w != self: w.pcls = ""



class List(ScrollArea):
    def _change(self, value):
        self.value = self.group.value
        self.send(CHANGE)

    def __init__(self, width, height, **params):
        params.setdefault('cls', 'list')
        self.table = table.Table(width=width)
        ScrollArea.__init__(self, self.table, width, height, no_hslider=True, **params)
        
        self.items = []
        
        g = group.Group()
        self.group = g
        g.connect(CHANGE,self._change,None)
        self.value = self.group.value = None
	
	self.add = self._add
	self.remove = self._remove
        
    def clear(self):
        self.items = []
        self.group = group.Group()
        self.group.connect(CHANGE,self._change,None)
        self.table.clear()
        self.set_vertical_scroll(0)
        self.blur(self.myfocus)
        
    def _add(self, label, image = None, value=None):
    	item = _List_Item(label,image=image,value=value)
        self.table.tr()
        self.table.add(item)
        self.items.append(item)
        item.group = self.group
        item.group.add(item)
        
    def _remove(self, item):
    	for i in self.items:
		if i.value == item: item = i
        if item not in self.items: return
        item.blur()
        self.items.remove(item)
        self.group.widgets.remove(item)
        self.table.remove_row(item.style.row)
        


class FileDialog(dialog.Dialog):
    def __init__(self, title_txt="File Browser", button_txt="Okay", cls="filedialog", path=None):
        if not path: self.curdir = os.getcwd()
        else: self.curdir = path
        import app
        self.dir_img = imagez.Image(app.App.app.theme.get(cls+'.folder', '', 'image'))
        td_style = {'padding_left': 4,
                    'padding_right': 4,
                    'padding_top': 2,
                    'padding_bottom': 2}
        self.title = labelz.Label(title_txt, cls=cls+".title.label")
        self.body = table.Table()
        self.list = List(width=350, height=150)
        self.input_dir = input.Input(cls=cls+".input")
        self.input_file = input.Input(cls=cls+".input")
        self._list_dir_()
        self.button_ok = button.Button(button_txt)
        self.body.tr()
        self.body.td(labelz.Label("Path", cls=cls+".label"), style=td_style, align=-1)
        self.body.td(self.input_dir, style=td_style)
        self.body.tr()
        self.body.td(labelz.Label("File", cls=cls+".label"), style=td_style, align=-1)
        self.body.td(self.input_file, style=td_style)
        self.body.td(self.button_ok, style=td_style)
        self.body.tr()
        self.body.td(self.list, colspan=3, style=td_style)
        self.list.connect(CHANGE, self._item_select_changed_, None)
        self.button_ok.connect(CLICK, self._button_okay_clicked_, None)
        self.value = None
        dialog.Dialog.__init__(self, self.title, self.body)
        
    def _list_dir_(self):
        self.input_dir.value = self.curdir
        self.input_dir.pos = len(self.curdir)
        self.input_dir.vpos = 0
        dirs = []
        files = []
        try:
            for i in os.listdir(self.curdir):
                if os.path.isdir(os.path.join(self.curdir, i)): dirs.append(i)
                else: files.append(i)
        except:
            self.input_file.value = "Opps! no access"
        if '..' not in dirs: dirs.append('..')
        dirs.sort()
        files.sort()
        for i in dirs:
            #item = ListItem(image=self.dir_img, text=i, value=i)
            self.list.add(i,image=self.dir_img,value=i)
        for i in files:
            #item = ListItem(image=None, text=i, value=i)
            self.list.add(i,value=i)
        self.list.resize()
        self.list.set_vertical_scroll(0)
        self.list.repaintall()

    def _item_select_changed_(self, arg):
        self.input_file.value = self.list.value
        fname = os.path.abspath(os.path.join(self.curdir, self.input_file.value))
        if os.path.isdir(fname):
            self.input_file.value = ""
            self.curdir = fname
            self.list.clear()
            self._list_dir_()

    def _button_okay_clicked_(self, arg):
        if self.input_dir.value != self.curdir:
            if os.path.isdir(self.input_dir.value):
                self.input_file.value = ""
                self.curdir = os.path.abspath(self.input_dir.value)
                self.list.clear()
                self._list_dir_()
        else:
            self.value = os.path.join(self.curdir, self.input_file.value)
	    self.send(CHANGE)
            self.close()